#!/usr/bin/perl
#
# LPAR2RRD agent daemon
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#
# set DEBUG=3 in etc/lpar2rrd.cfg to get verbose logging in etc/daemon.out
#

use strict;
use RRDp;
use Date::Parse;
use IO::Socket::INET;
use File::Copy;
use File::Compare;


my $version="$ENV{version}";
my $rrdtool = $ENV{RRDTOOL};
my $DEBUG = $ENV{DEBUG};
#$DEBUG=3;
my $pic_col = $ENV{PICTURE_COLOR};
my $STEP = $ENV{SAMPLE_RATE};
my $port = $ENV{LPAR2RRD_AGENT_DAEMON_PORT};
my $basedir = $ENV{INPUTDIR};
my $tmpdir = "$basedir/tmp";
if (defined $ENV{TMPDIR}) {
  $tmpdir = $ENV{TMPDIR};
}
my $wrkdir = "$basedir/data";

my $listen = $ENV{LPAR2RRD_AGENT_DAEMON_IP};
if ($listen eq '') {
  $listen = "0.0.0.0";
}

`echo "$$" > "$basedir/tmp/lpar2rrd-daemon.pid"`;

# standard data retentions
my $one_minute_sample=86400;
my $five_mins_sample=25920;
my $one_hour_sample=4320;
my $five_hours_sample=1734;
my $one_day_sample=1080;
load_retentions ($STEP,$basedir);

my $lpar = ""; # global one as well to do not chek out cpu.cfg every time for lpar name

if ( $port eq '' ) {
  $port=8162; # IANA LPAR2RRD registered port
}

# Input limits for RRDTool files 
# It must be there to avoid peaks caused by reseting counters
my $INPUT_LIMIT_PGS = 500000;       # 500MB/sec
my $INPUT_LIMIT_LAN = 1250000000;   # 10Gbits/sec
my $INPUT_LIMIT_PCK_LAN = 1250000;  # Packets
my $INPUT_LIMIT_SEA = 1250000000;   # 10Gbits/sec
my $INPUT_LIMIT_PCK_SEA = 1250000;  # Packets
my $INPUT_LIMIT_SAN1 = 1000000000;  # 8GBits/sec
my $INPUT_LIMIT_SAN2 = 100000;      # IOPS


# flush after every write
$| = 1;

my ($socket,$client_socket);
my ($peeraddress,$peerport);

# creating object interface of IO::Socket::INET modules which internally does
# socket creation, binding and listening at the specified port address.
$socket = new IO::Socket::INET (
LocalHost => $listen,
LocalPort => $port,
Proto => 'tcp',
Listen => 100,
Reuse => 1
) || error("ERROR in Socket Creation : $!\n".__FILE__.":".__LINE__) && exit(1);

setsockopt($socket, SOL_SOCKET, SO_REUSEADDR, 1) || error("Can't set socket option to SO_REUSEADDR $!".__FILE__.":".__LINE__) && exit(1);

print_it ("LPAR2RRD server waiting for client connections on $listen:$port");

# start RRD via a pipe
RRDp::start "$rrdtool";
#$RRDp::error_mode = 'catch'; 


my $error_flag = 0;
my $protocol_version = 0;
my $peer_address = "";
my $name = "";
my $peer_port = "";
my $cpu_hz_glo = 0; # keep CPU HZ and save it just once per a run

while(1)
{

  if ( $error_flag == 0 ) {
    $cpu_hz_glo = 0;  # reset it for every connection
    # normal behaviour, no error detected 
    # waiting for new client connection.
    $client_socket = $socket->accept();

    # get the host and port number of newly connected client.
    $peer_address = $client_socket->peerhost();
    $name = gethostbyaddr($peer_address, AF_INET );
    $peer_port = $client_socket->peerport();

    # flush after every write
    $| = $client_socket;
  }

  # use alert here to be sure it does not hang due to 1 hanging connection
  my $timeout = 180; # it should be enough
  eval {
    my $act_time = localtime();
    local $SIG{ALRM} = sub {die "$act_time: died in SIG ALRM: $name ($peer_address)";};
    alarm($timeout);

    # no fork so far, it would be quite CPU expensive, as new process, new rrdtool ....
    read_client($client_socket,$peer_address,$name,$peer_port,$error_flag);

    alarm (0);
  };

  chomp ($@);
  if ($@) {
    if ($@ =~ /died in SIG ALRM/) {
      error ("Client connection timed out after : $timeout seconds - client: $name ($peer_address)".__FILE__.":".__LINE__);
    }
    else {
      if ($@ =~ m/illegal attempt to update/ ) {
        # when a problem with the insert time then confirm the time as ok
        my $time_err = $@;
        $time_err =~ s/^.*to update using time //;
        $time_err =~ s/ when last update time is.*$//;
        error ("Client comunication failed - client: $name ($peer_address): $@ : sending ok time anyway : $time_err :");
        print "Response to the agent: $time_err\n" if $DEBUG > 2;
        print $client_socket "$time_err\n";
        $error_flag = 1;
        next; # go to next loop to read same client data untill the end
      }
      else {
        error ("Client comunication failed - client: $name ($peer_address): $@ ");
      }
    }
  }
  close($client_socket);
  $error_flag = 0;
}

$socket->close();

# close RRD pipe
RRDp::end;

exit (0);


sub read_client
{
  my $client_socket = shift;
  my $peer_address = shift;
  my $name = shift;
  my $peer_port = shift;
  my $error_flag = shift;
  my $act_time = localtime();

  if ( $error_flag == 0 ) {
    # normal behaviour where there was not an error and not necessary to read the client data untill the end
    # read protocol version
    my $protocol_version_tmp = <$client_socket>;
    chomp($protocol_version_tmp);
    $protocol_version = substr($protocol_version_tmp,0,3); # must be Global one
    $protocol_version = $protocol_version * 10;
    #print "conn from:$name ($peer_address) : port:$peer_port protocol: $protocol_version\n" if $DEBUG > 2;

    if ( $protocol_version eq '' || isdigit($protocol_version) == 0 ) {
      print $client_socket "Protocol error: $protocol_version\n";
      $protocol_version =~s/([^A-Za-z0-9\+-_])/sprintf("%%%02X",ord($1))/seg; # to make it readable in logs
      error("Received bad conn from:$name ($peer_address) : port:$peer_port : $protocol_version ".__FILE__.":".__LINE__);
      return 1;
    }
  }

  print "$act_time: Received conn  : $name ($peer_address) :$peer_port : $protocol_version\n" if $DEBUG > 2;

  # read data, all records
  my $last_rec = 0; # it must be zeroed here for each new connection, but not for each row in the connection
  $lpar = ""; # global one as well to do not chek out cpu.cfg every time for lpar name
  while ( my $data = <$client_socket> ) { 
    chomp ($data);
    if ( $data eq '' || length($data) < 20 ) {
      $data =~s/([^A-Za-z0-9\+-_])/sprintf("%%%02X",ord($1))/seg; # to make it readable in logs
      $protocol_version =~s/([^A-Za-z0-9\+-_])/sprintf("%%%02X",ord($1))/seg; # to make it readable in logs
      error("Received bad data from:$name ($peer_address) : port:$peer_port : protocol:$protocol_version : data:$data ".__FILE__.":".__LINE__);
      last;
    }
    #print "001: $last_rec - $data";


    if ( $protocol_version < 20 ) {
      $last_rec = store_data_10($data,$last_rec,$protocol_version,$peer_address);
    }
    else {
      $last_rec = store_data($data,$last_rec,$protocol_version,$peer_address);
    }

    # write answer (just time as confirmation that it has been stored in the DB)
    print "Response to the agent: $last_rec\n" if $DEBUG > 2;
    print $client_socket "$last_rec\n";

    if ($last_rec == 0) {
      last; # end as a fatal error appeared
    }
  }
  close($client_socket); 

  return 1;
}


# returns 0 - fatal error, original $time as OK 
# $lpar is global variable where is the lpar name which was found out based on lpar_id
#  --> it is done once per a connection to do not do it for each record

sub store_data     
{
  my $data = shift;
  my $last_rec = shift;
  my $protocol_version = shift;
  my $peer_address = shift;
  my $en_last_rec = $last_rec;
  my $act_time = localtime();

  #example of $data here incl item names for docum purpose ! data is one line !
 # 8233-E8B*5383FP:BSRV21LPAR5-pavel:5:1392202714:Wed Feb 12 11:58:34 2014:::::
 # mem:::3932160:3804576:127584:1267688:2369200:1435376:
 # pgs:::0:0:4096:1:::
 # lan:en2:172.31.241.171:1418275448:444418173:::::
 # lan:en4:172.31.216.135:22069646900:1249033690:::::
 # san:fcs0:0xC050760329FB00C0:24671446454:16462307328:798417:1908861:::
 # san:fcs1:0xC050760329FB00C2:678475916:1048829952:22837:13854::
 # cpu:::1:2:0:0::

 # [server|serial]:lpar:lpar_id:time_stamp_unix:time_stamp_text:  #mandatory
 #      future_usage1:future_usage2:future_usage3:future_usage4:  #mandatory  
  # other non-mandatory items depends on machine HW and SW
 # :mem:::$size:$inuse:$free:$pin:$in_use_work:$in_use_clnt
 # :pgs:::$page_in:$page_out:$paging_space:$pg_percent::
 # :lan:$en:$inet:$transb:$recb::::
 # :san:$line:$wwn:$inpb:$outb:$inprq:$outrq::
 # :ame:::$comem:$coratio:$codefic:::
 # :cpu:::$entitled:$cpu_sy:$cpu_us:$cpu_wa::
 # :sea:$back_en:$en:$transb:$recb::::
 
 # example of data from VIOS    one line
 # 8233-E8B*53840P:ASRV12VIOS1:1:1392643440:Mon Feb 17 14:24:00 2014:::::
 # mem:::3145728:2038184:1107544:1588236:1921944:116240:
 # pgs:::801001:899921:2048:9:::sea:ent1:ent28:0:4760483564:::::
 # sea:ent2:ent29:0:4760483564:::::sea:ent3:ent30:0:4760473500:::::
 # sea:ent4:ent31:0:4760483684:::::sea:ent5:ent32:0:4751724058:::::
 # sea:ent6:ent33:0:4751717266:::::sea:ent7:ent34:0:4751714234:::::
 # sea:ent8:ent35:0:4751723938:::::sea:ent0:ent36:26874:6254173249:::::
 # lan:en27:172.31.216.71:625267876023:1715959232831:::::
 # san:fcs0:0x20000000C9F0B32A:599641385932:1765316803092:184200748:463211964:::
 # san:fcs1:0x20000000C9F0B32B:8576634954676:281457862563932:146970574:8165833668:::
 # san:fcs2:0x20000000C9F0B3D6:17430139432091:25323018062172:429245470:720164002:::
 # san:fcs3:0x20000000C9F0B3D7:823034884094:3439309033364:310735998:887024242::

  my $server = "";
  my $lpar_name = "";
  my $time = "";
  my $db_write_time = ""; 
  my $lpar_id = -1;

 # items count test

  my @datar = split(":",$data."sentinel");
  my $datar_len = @datar;
  my @datar = split(":",$data."sentinel");
  my $datar_len = @datar;

  if (($datar_len % 9) != 2 ) {
    if ($datar_len > 3 && ! $datar[3] eq '' && isdigit($datar[3])) {
      # try to skip over that error and send back right response == saved instead of stucking here for ever
      error("$peer_address: not correct items count: $datar_len, $data : try return ok ($datar[3])".__FILE__.":".__LINE__);
      return ($datar[3]);
    }
    else {
      error("$peer_address: not correct items count: $datar_len, $data :".__FILE__.":".__LINE__);
      return (0);
    }
  }
 
 # processing mandatory fieds means:
  # find out lpar name from lpar_id
  # prepare db write time
  # prepare space proof server and lpar names
 
  $server = $datar[0]; 
  $lpar   = $datar[1];
  $lpar_id = $datar[2];
  $time    = $datar[3];
  my $cpu_hz = $datar[7];
  $server  =~  s/====double-colon=====/:/g;
  $lpar    =~ s/=====double-colon=====/:/g;

  if ( $server eq '' ) {
    if ($datar_len > 3 && ! $datar[3] eq '' && isdigit($datar[3])) {
      # try to skip over that error and send back right response == saved instead of stucking here for ever
      error("server is null: $datar_len, $data : $server : $lpar : $lpar_id : $time : try return ok ($datar[3]) ".__FILE__.":".__LINE__);
      return ($datar[3]);
    }
    else {
      error("server is null: $datar_len, $data : $server : $lpar : $lpar_id : $time ".__FILE__.":".__LINE__);
      return (0);
    }
  }

  $cpu_hz_glo = save_cpu_ghz ($cpu_hz_glo,$cpu_hz,$server,$wrkdir);

 
  # a bit trick how to find a symlink, there was a bug before 3.70 and agents transfered only 6 chars of serial (instead of 7)
  # in original constructions it is not a problem due to a "*" inside the link and shell usage <>
  my @servers = <$wrkdir/$server>;
  my $found = 0;
  foreach my $file (@servers) {
    if ( -l "$wrkdir/$server" ) {
      $found = 1;
      last;
    }
  }
  if ( $found = 0 ) {
    # sym link does not exist --> server is either not registered yet or full lpar without the HMC
    my $NOSERVER_SUFFIX = "--unknown";
    my $NOHMC = "no_hmc";
    if ( ! -d "$wrkdir/$server$NOSERVER_SUFFIX" ) {
      mkdir("$wrkdir/$server$NOSERVER_SUFFIX", 0755) || error(" Cannot mkdir $wrkdir/$server: $!".__FILE__.":".__LINE__) && return 0;
    }
    symlink("$wrkdir/$server$NOSERVER_SUFFIX","$wrkdir/$server") || error(" Cannot ln -s $wrkdir/$server$NOSERVER_SUFFIX $wrkdir/$server: $!".__FILE__.":".__LINE__) && return 0;
    if ( ! -d "$wrkdir/$server/$NOHMC" ) {
      mkdir("$wrkdir/$server/$NOHMC", 0755) || error(" Cannot mkdir $wrkdir/$server/$NOHMC: $!".__FILE__.":".__LINE__) && return 0;
    }
    my $ltime = localtime();
    print_it ("new server has been found and registered: $server (lpar=$lpar)");
    #if ( -f "$wrkdir/$server/$NOHMC/$lpar.rrh" ) {
    #  # trick how to say lpar2rrd that there is a "std" lpar
    #  `touch ("$wrkdir/$server/$NOHMC/$lpar.rrh")`;
    #}
    # --> done in install_html.sh
    if ( -f "$wrkdir/$server/$NOHMC/pool.rrh" ) {
      # trick how to say lpar2rrd that there is a "std" server
      `touch ("$wrkdir/$server/$NOHMC/pool.rrh")`;
    }
    touch ("");
  }

  # find out lpar name from lpar_id if it has not been done yet
  # oit is a primary method to get the lpar name
  if ( $lpar_id != -1 ) {
    my $lpar_name_id = find_lpar_name($server,$lpar_id);
    if ( ! $lpar_name_id eq '' ) {
      $lpar = $lpar_name_id;  # lpar name from lparstat -i --> it does not have to be actual,
                              # linux on power does not provide lpar name at all, only hostname
    }
  }

  if ( $lpar eq '' && ! $lpar_name eq '' ) {
    # just make sure if lpar-id fails somehow then use transferred $lpar_name
    $lpar = $lpar_name;
  }

  if ( $lpar eq '' ) {
    error("$peer_address: lpar name has not been found for client: $peer_address , server:$server, lpar_id:$lpar_id".__FILE__.":".__LINE__);
    return (0);
  }

  # prepare db write time for DB saving data from time_stamp_text
  #$time = str2time($datar[4].":".$datar[5].":".$datar[6]);
  # do not use time_stamp_text due to TZ problems
  if ($time < 1344279720 ) {   #  old than Year 2012 not possible
    error("$peer_address: bad data time: server:$server, lpar_id:$lpar_id : ignore it".__FILE__.":".__LINE__);
    return ($time);
  }

  print "$data" if $DEBUG > 2;

  my $lpar_real = $lpar;
  $lpar_real =~ s/\//&&1/g;

  my $rrd_file = "";
  my $lpar_space = $lpar_real;
  if ( $lpar_real =~ m/ / ) {
    $lpar_space = "\"".$lpar_real."\""; # it must be here to support space with lpar names
  }
  my $server_space = $server;
  if ( $server =~ m/ / ) {
    $server_space = "\"".$server."\""; # it must be here to support space with server names
  }

  #
  # cycle for non-mandatory items
  #

  my $cycle_var = 11; # is a pointer to data array
  while ($datar[$cycle_var]) {
    my $db_name = $datar[$cycle_var]; #prepare db name
    if ($db_name eq "lan" || $db_name eq "san" || $db_name eq "sea" ) {
        $db_name .= "-".$datar[$cycle_var+1];
    }
    $db_name .= "\.mmm";
    $rrd_file = "";
    my @files = <$wrkdir/$server_space/*/$lpar_space/$db_name>;
    foreach my $rrd_file_tmp (@files) {
      chomp($rrd_file_tmp);
      $rrd_file = $rrd_file_tmp;
      last;
    }
    print "name is $db_name rrd is $rrd_file\n" if $DEBUG >2;
    if ( ! $rrd_file eq '' ) {
      my $filesize = -s "$rrd_file";
      if ( $filesize == 0 ) {
        # when a FS is full then it creates 0 Bytes rrdtool files what is a problem, delete it then
        error("0 size rrd file: $rrd_file  - delete it".__FILE__.":".__LINE__);
        unlink ("$rrd_file") || error ("Cannot rm $rrd_file : $!".__FILE__.":".__LINE__);
        $rrd_file = ""; # force to create a new one
      }
    }

    print "$act_time: Updating       : $server_space:$lpar_space - $rrd_file\n" if $DEBUG > 2;
    if ( $rrd_file eq '' ) {
      if ( create2_rrd($server,$lpar_real,$time,$server_space,$lpar_space,$db_name) == 0 ) {
        return 0;
      }
      @files = <$wrkdir/$server_space/*/$lpar_space/$db_name>;
      foreach my $rrd_file_tmp (@files) {
        chomp($rrd_file_tmp);
        $rrd_file = $rrd_file_tmp;
        last;
      }
    }
    if ($last_rec == 0 ) {
      # construction against crashing daemon Perl code when RRDTool error appears
      # this does not work well in old RRDTOool: $RRDp::error_mode = 'catch';
      # construction is not too costly as it runs once per each load
      eval {
        RRDp::cmd qq(last "$rrd_file" );
        my $last_rec_rrd = RRDp::read;
        chomp ($$last_rec_rrd);
        $last_rec = $$last_rec_rrd;
      };
      if ($@) {
        rrd_error($@.__FILE__.":".__LINE__,$rrd_file);
        return 0;
      }
    }

    if ( ($last_rec + $STEP/2) >= $time ) {
      #error("$server:$lpar : last rec : $last_rec + $STEP/2 >= $time, ignoring it ...".__FILE__.":".__LINE__);
      print "$act_time: Updating 4 : $last_rec : $time : $rrd_file\n" if $DEBUG > 2;
      return $time; # returns original time, not last_rec
	  # --> no, no, it is not wrong, just ignore it!
    }

  #
  # files update 
  #

  # alias case structure
    print "$act_time: case struc : $rrd_file : $cycle_var : $datar[$cycle_var]\n" if $DEBUG > 2;
    my $answer = "";
    my $nan = "U";
    {
    "mem" eq $datar[$cycle_var] && do {
        RRDp::cmd qq(update "$rrd_file" $time:$datar[$cycle_var+3]:$datar[$cycle_var+4]:$datar[$cycle_var+5]:$datar[$cycle_var+6]:$datar[$cycle_var+7]:$datar[$cycle_var+8]);
        $answer = RRDp::read;
        last;
        };
    "pgs" eq $datar[$cycle_var] && do {
        RRDp::cmd qq(update "$rrd_file" $time:$datar[$cycle_var+3]:$datar[$cycle_var+4]:$datar[$cycle_var+5]:$datar[$cycle_var+6]);
        $answer = RRDp::read;
        last;
        };
    "lan" eq $datar[$cycle_var] && do {
        RRDp::cmd qq(update "$rrd_file" $time:$datar[$cycle_var+3]:$datar[$cycle_var+4]:$nan:$nan);
        $answer = RRDp::read;
        write_lansancfg ( $rrd_file, $datar[$cycle_var+2], $time); 
        last;
        };
    "san" eq $datar[$cycle_var] && do {
        RRDp::cmd qq(update "$rrd_file" $time:$datar[$cycle_var+3]:$datar[$cycle_var+4]:$datar[$cycle_var+5]:$datar[$cycle_var+6]);
        $answer = RRDp::read;
        write_lansancfg ( $rrd_file, $datar[$cycle_var+2], $time);
        last;
        };
    "ame" eq $datar[$cycle_var] && do {
        RRDp::cmd qq(update "$rrd_file" $time:$datar[$cycle_var+3]:$datar[$cycle_var+4]:$datar[$cycle_var+5]);
        $answer = RRDp::read;
        last;
        };
    "cpu" eq $datar[$cycle_var] && do {
        RRDp::cmd qq(update "$rrd_file" $time:$datar[$cycle_var+3]:$datar[$cycle_var+4]:$datar[$cycle_var+5]:$datar[$cycle_var+6]);
        $answer = RRDp::read; 
        last;
        };
    "sea" eq $datar[$cycle_var] && do {
        RRDp::cmd qq(update "$rrd_file" $time:$datar[$cycle_var+3]:$datar[$cycle_var+4]:$nan:$nan);
        $answer = RRDp::read;
        write_lansancfg ( $rrd_file, $datar[$cycle_var+2], $time);
        last;
        };
    error ("Unknown item from agent : $datar[$cycle_var] ($datar[$cycle_var+1]:$datar[$cycle_var+2]:$datar[$cycle_var+3]:$datar[$cycle_var+4]) ".__FILE__.":".__LINE__); 
    } 
    print "000 : $datar[$cycle_var] $cycle_var : $time:$datar[$cycle_var+3]:$datar[$cycle_var+4]:$datar[$cycle_var+5]:$datar[$cycle_var+6] \n" if $DEBUG > 2;
    #   my $answer = RRDp::read;
    if ( ! $$answer eq '' && $$answer =~ m/ERROR/ ) {
      error (" updating $server:$lpar : $rrd_file : $$answer".__FILE__.":".__LINE__);
      if ( $$answer =~ m/is not an RRD file/ ) {
        (my $err,  my $file, my $txt) = split(/'/,$$answer);
        error ("Removing as it seems to be corrupted: $file".__FILE__.":".__LINE__);
        unlink("$file") || error ("Cannot rm $file : $!".__FILE__.":".__LINE__);
      }
      return 0;
    }
    # prepare next cycle - always skips 9 atoms
    $cycle_var = $cycle_var + 9;
  }
  #print "001 : $datar[$cycle_var] $cycle_var\n" if $DEBUG > 2;
  print "finish storing data from agent\n" if $DEBUG > 2;
  return $time;             # return time of last record
}

sub write_lansancfg {
 
   # write cfg files containing IP (for enXX) & WWN (for fcsXX) & name for sea
   my $rrd_file = shift;
   my $enip = shift;       # cfg info
   my $time = shift;

   print "sub write cfg file, rrd_file  $rrd_file\n" if $DEBUG > 2;
   $rrd_file =~ s/\.mmm/\.cfg/g;
   my $size =  -s $rrd_file;
   if ( ! -f $rrd_file || $size == 0 ) {
     open(FW, "> $rrd_file") || error("Can't open $rrd_file : $!".__FILE__.":".__LINE__) && return 0;
     print FW "$enip\n";
     close (FW);
   }
   else { 
     my $timem = ((stat($rrd_file))[9]);
     if (( $time - $timem) > 86400 ){
       open(FW, "> $rrd_file") || error("Can't open $rrd_file : $!".__FILE__.":".__LINE__) && return 0;
       print FW "$enip\n";
       close (FW);
     }
   } 
 }

sub create2_rrd
{
  my $server = shift;
  my $lpar = shift;
  my $time = shift;
  my $server_space = shift;
  my $lpar_space = shift;
  my $db_name = shift;

  $time = $time - $STEP; # start time lower than actual one being updated

  my $no_time = $STEP * 7; # says the time interval when RRDTOOL considers a gap in input data
  my $act_time = localtime();
  my $found = 0;
  my $rrd_dir = "";

  print "creating $db_name\n" if $DEBUG >2;
  my @files = <$wrkdir/$server_space/*>;
  foreach my $rrd_dir_tmp (@files) {
    chomp($rrd_dir_tmp);
    if ( -d $rrd_dir_tmp ) {
      $found = 1;
      $rrd_dir = $rrd_dir_tmp;
      last;
    }
  }

  if ( $found == 0 ) {
    error ("Could not found a HMC in : $wrkdir/$server".__FILE__.":".__LINE__) && return 0;
  }
  if ( ! -d "$rrd_dir/$lpar/" ) {
     print_it ("mkdir          : $rrd_dir/$lpar/") if $DEBUG;
     mkdir("$rrd_dir/$lpar/") || error("Cannot mkdir $rrd_dir/$lpar/: $!".__FILE__.":".__LINE__) && return 0;
     touch ("");
  }

  my $rrd = "$rrd_dir/$lpar/$db_name";
  print "$act_time: RRD create     : $rrd\n" if $DEBUG > 2;
  {
  $db_name =~ /mem/ && do {

     if ( -f "$rrd_dir/$lpar.mmm" ) {
        return conv_lpar("$rrd_dir/$lpar.mmm",$server,$lpar,"$rrd_dir/$lpar",$tmpdir );
     };

     RRDp::cmd qq(create "$rrd"  --start "$time"  --step "$STEP"
      "DS:size:GAUGE:$no_time:0:102400000000"
      "DS:nuse:GAUGE:$no_time:0:102400000000"
      "DS:free:GAUGE:$no_time:0:102400000000"
      "DS:pin:GAUGE:$no_time:0:102400000000"
      "DS:in_use_work:GAUGE:$no_time:0:102400000000"
      "DS:in_use_clnt:GAUGE:$no_time:0:102400000000"
      "RRA:AVERAGE:0.5:1:$one_minute_sample"
      "RRA:AVERAGE:0.5:5:$five_mins_sample"
      "RRA:AVERAGE:0.5:60:$one_hour_sample"
      "RRA:AVERAGE:0.5:300:$five_hours_sample"
      "RRA:AVERAGE:0.5:1440:$one_day_sample"
      );
     last
     };
  $db_name =~ /pgs/ && do {

     RRDp::cmd qq(create "$rrd"  --start "$time" --step "$STEP"
      "DS:page_in:COUNTER:$no_time:0:$INPUT_LIMIT_PGS"
      "DS:page_out:COUNTER:$no_time:0:$INPUT_LIMIT_PGS"
      "DS:paging_space:GAUGE:$no_time:0:U"
      "DS:percent:GAUGE:$no_time:0:100"
      "RRA:AVERAGE:0.5:1:$one_minute_sample"
      "RRA:AVERAGE:0.5:5:$five_mins_sample"
      "RRA:AVERAGE:0.5:60:$one_hour_sample"
      "RRA:AVERAGE:0.5:300:$five_hours_sample"
      "RRA:AVERAGE:0.5:1440:$one_day_sample"
      );
     last
     };
  $db_name =~ /lan/ && do {

     RRDp::cmd qq(create "$rrd"  --start "$time" --step "$STEP"
      "DS:recv_bytes:COUNTER:$no_time:0:$INPUT_LIMIT_LAN"
      "DS:trans_bytes:COUNTER:$no_time:0:$INPUT_LIMIT_LAN"
      "DS:recv_packets:COUNTER:$no_time:0:$INPUT_LIMIT_PCK_LAN"
      "DS:trans_packets:COUNTER:$no_time:0:$INPUT_LIMIT_PCK_LAN"
      "RRA:AVERAGE:0.5:1:$one_minute_sample"
      "RRA:AVERAGE:0.5:5:$five_mins_sample"
      "RRA:AVERAGE:0.5:60:$one_hour_sample"
      "RRA:AVERAGE:0.5:300:$five_hours_sample"
      "RRA:AVERAGE:0.5:1440:$one_day_sample"
      );
     last
     };
  $db_name =~ /san/ && do {

     RRDp::cmd qq(create "$rrd"  --start "$time" --step "$STEP"
      "DS:recv_bytes:COUNTER:$no_time:0:$INPUT_LIMIT_SAN1"
      "DS:trans_bytes:COUNTER:$no_time:0:$INPUT_LIMIT_SAN1"
      "DS:iops_in:COUNTER:$no_time:0:$INPUT_LIMIT_SAN2"
      "DS:iops_out:COUNTER:$no_time:0:$INPUT_LIMIT_SAN2"
      "RRA:AVERAGE:0.5:1:$one_minute_sample"
      "RRA:AVERAGE:0.5:5:$five_mins_sample"
      "RRA:AVERAGE:0.5:60:$one_hour_sample"
      "RRA:AVERAGE:0.5:300:$five_hours_sample"
      "RRA:AVERAGE:0.5:1440:$one_day_sample"
      );
     last
     };
  $db_name =~ /ame/ && do {

     RRDp::cmd qq(create "$rrd"  --start "$time" --step "$STEP"
      "DS:ame_mem:GAUGE:$no_time:0:U"
      "DS:ame_ratio:GAUGE:$no_time:0:1000"
      "DS:ame_deficit:GAUGE:$no_time:0:U"
      "RRA:AVERAGE:0.5:1:$one_minute_sample"
      "RRA:AVERAGE:0.5:5:$five_mins_sample"
      "RRA:AVERAGE:0.5:60:$one_hour_sample"
      "RRA:AVERAGE:0.5:300:$five_hours_sample"
      "RRA:AVERAGE:0.5:1440:$one_day_sample"
      );
     last
     };
  $db_name =~ /cpu/ && do {
   
     RRDp::cmd qq(create "$rrd"  --start "$time" --step "$STEP"
      "DS:entitled:GAUGE:$no_time:0:U"
      "DS:cpu_sy:GAUGE:$no_time:0:U"
      "DS:cpu_us:GAUGE:$no_time:0:U"
      "DS:cpu_wa:GAUGE:$no_time:0:U"
      "RRA:AVERAGE:0.5:1:$one_minute_sample"
      "RRA:AVERAGE:0.5:5:$five_mins_sample"
      "RRA:AVERAGE:0.5:60:$one_hour_sample"
      "RRA:AVERAGE:0.5:300:$five_hours_sample"
      "RRA:AVERAGE:0.5:1440:$one_day_sample"
      );
     last
     };
  $db_name =~ /sea/ && do {

     RRDp::cmd qq(create "$rrd"  --start "$time" --step "$STEP"
      "DS:recv_bytes:COUNTER:$no_time:0:$INPUT_LIMIT_SEA"
      "DS:trans_bytes:COUNTER:$no_time:0:$INPUT_LIMIT_SEA"
      "DS:recv_packets:COUNTER:$no_time:0:$INPUT_LIMIT_PCK_SEA"
      "DS:trans_packets:COUNTER:$no_time:0:$INPUT_LIMIT_PCK_SEA"
      "RRA:AVERAGE:0.5:1:$one_minute_sample"
      "RRA:AVERAGE:0.5:5:$five_mins_sample"
      "RRA:AVERAGE:0.5:60:$one_hour_sample"
      "RRA:AVERAGE:0.5:300:$five_hours_sample"
      "RRA:AVERAGE:0.5:1440:$one_day_sample"
      );
     last
     };
  error ("Really Unknown item from agent : $db_name ".__FILE__.":".__LINE__) && return 0;
  }

  my $answer = RRDp::read;
  if ( $$answer ) {
    error ("unable to create $rrd: answer: $$answer".__FILE__.":".__LINE__);
    return 0;
  }

  # create lpar directory and file hard link into the other HMC if there is dual HMC setup
  my $rrd_dir_base = basename($rrd_dir);
  foreach my $rrd_dir_new (@files) {
    chomp($rrd_dir_new);
    my $rrd_dir_new_base = basename($rrd_dir_new);
    if ( -d $rrd_dir_new && $rrd_dir_new_base !~ m/^$rrd_dir_base$/ ) {
      if ( ! -d "$rrd_dir_new/$lpar/" ) {
        print_it ("mkdir dual     : $rrd_dir_new/$lpar/") if $DEBUG ;
        mkdir("$rrd_dir_new/$lpar/") || error("Cannot mkdir $rrd_dir_new/$lpar/: $!".__FILE__.":".__LINE__) && return 0;
      }
    print_it ("hard link      : $rrd --> $rrd_dir_new/$lpar/$db_name") if $DEBUG ;
    my $rrd_link_new = "$rrd_dir_new/$lpar/$db_name";
    unlink("$rrd_dir_new/$lpar/$db_name");   # for sure  
    link ($rrd,"$rrd_dir_new/$lpar/$db_name") || error("Cannot link $rrd:$rrd_dir_new/$lpar/$db_name : $!".__FILE__.":".__LINE__) && return 0;
    touch ("");

    # same for .cfg files
    $rrd =~ s/mmm$/cfg/;
    $db_name =~ s/mmm$/cfg/;
    if (($db_name =~ m/^lan-/ || $db_name =~ m/^san-/ || $db_name =~ m/^sea-/ ) && ! -f $rrd && ! -f "$rrd_dir_new/$lpar/$db_name" ) {
     `touch $rrd`;
      link ($rrd,"$rrd_dir_new/$lpar/$db_name") || error("Cannot link $rrd:$rrd_dir_new/$lpar/$db_name : $!".__FILE__.":".__LINE__) && return 0;
    }
          
    }
  } 
  return 1;
}

sub load_retentions
{
  my $step=shift;
  my $basedir = shift;

  # standards
  $one_minute_sample=86400;
  $five_mins_sample=25920;
  $one_hour_sample=4320;
  $five_hours_sample=1734;
  $one_day_sample=1080;

  if ( ! -f "$basedir/etc/retention.cfg" ) {
    # standard retentions in place
    return 0;
  }

  # extra retentions are specifiled in $basedir/etc/retention.cfg
  open(FH, "< $basedir/etc/retention.cfg") || error ("Can't read from: $basedir/etc/retention.cfg: $!");

  my @lines = <FH>;
  foreach my $line (@lines) {
    chomp ($line);
    if ( $line =~ m/^1min/ ) {
      (my $trash, $one_minute_sample) = split(/:/,$line);
    }
    if ( $line =~ m/^5min/ ) {
      (my $trash, $five_mins_sample) = split(/:/,$line);
    }
    if ( $line =~ m/^60min/ ) {
      (my $trash, $one_hour_sample) = split(/:/,$line);
    }
    if ( $line =~ m/^300min/ ) {
      (my $trash, $five_hours_sample) = split(/:/,$line);
    }
    if ( $line =~ m/^1440min/ ) {
      (my $trash, $one_day_sample) = split(/:/,$line);
    }
  }

  close (FH);

  $step = $step / 60;
  $one_minute_sample = $one_minute_sample / $step;
  $five_mins_sample = $five_mins_sample / $step;
  $one_hour_sample = $one_hour_sample / $step;
  $five_hours_sample = $five_hours_sample / $step;
  $one_day_sample = $one_day_sample / $step;

  return 1;
}

sub basename {
  my $full = shift;
  my $out = "";

  # basename without direct function
  my @base = split(/\//,$full);
  foreach my $m (@base) {
    $out = $m;
  }

  return $out;
}

sub rrd_error
{
  my $err_text = shift;
  my $rrd_file = shift;
  my $tmpdir = "$basedir/tmp";
  if (defined $ENV{TMPDIR}) {
    $tmpdir = $ENV{TMPDIR};
  }

  chomp ($err_text);

  if ( $err_text =~ m/ERROR:/ && ! $rrd_file eq '' ) {
    # copy of the corrupted file into "save" place and remove the original one
    copy ("$rrd_file","$tmpdir/") || error("Cannot: cp $rrd_file $tmpdir/: $!".__FILE__.":".__LINE__);
    unlink("$rrd_file") || error("Cannot rm $rrd_file : $!".__FILE__.":".__LINE__);
    error("$err_text, moving it into: $tmpdir/");
  }
  else {
    error("$err_text");
  }
  return 0;
}

# extends fcsxx file for iops in-out

sub fcs_iops {
    my $rrd_file = shift;
    my $server = shift;
    my $lpar = shift;
    my $ent = shift;
#  read dumped file, defining new ds,
   my $f_dumped = "/tmp/rrd_dump";
   unlink($f_dumped); #in any case
   (! `rrdtool dump $rrd_file $f_dumped`) || error("Cannot dump $rrd_file to $f_dumped: $!".__FILE__.":".__LINE__) && return 0;
  open (DF_IN,"< $f_dumped") || error("Cannot open for reading $f_dumped: $!".__FILE__.":".__LINE__) && return 0;
  my @lines = <DF_IN>;
  close (DF_IN);
  unlink($f_dumped);
  $f_dumped = "/tmp/rrd_dump_q";
  open (DF_OUT, "> $f_dumped") || error("Cannot open for writing $f_dumped: $!".__FILE__.":".__LINE__) && return 0;

  my (@ds1, @ds2, $line );
  while ( $line = shift @lines) {
     print DF_OUT "$line";
     last if ( $line =~ "<ds>" );
  }
#print "$lines[0],$lines[1],$lines[2],$lines[3]\n";
  $ds1[0] = $line;
  for (my $i = 1; $i <= 11; $i++) {
      $line = shift @lines;
      print DF_OUT "$line";
      if ( $line =~ "recv_bytes" ) {
         $line =~ s/recv_bytes/iops_in/;
         $ds1[$i] = $line;
         next;
      }
      if ( $line =~ "<min>" ) {
         $ds1[$i] = "           <min> 0.0000000000e+00 </min>\n";
         next;
      }
      if ( $line =~ "<value>" ) {
         $ds1[$i] = "           <value> 0.0000000000e+00 </value>\n";
         next;
      }
      $ds1[$i] = $line;
      next;
  }
  while ( $line = shift @lines) {
     print DF_OUT "$line";
     last if ( $line =~ "<ds>" );
  }
  $ds2[0] = $line;
  for (my $i=1; $i <= 11; $i++) {
      $line = shift @lines;
      print DF_OUT "$line";
      if ( $line =~ "trans_bytes" ) {
         $line =~ s/trans_bytes/iops_out/;
         $ds2[$i] = $line;
         next;
      }
      if ( $line =~ "<min>" ) {
         $ds2[$i] = "           <min> 0.0000000000e+00 </min>\n";
         next;
      }
      if ( $line =~ "<value>" ) {
         $ds2[$i] = "           <value> 0.0000000000e+00 </value>\n";
         next;
      }
      $ds2[$i] = $line;
      next;
  }
# print next two ds
  print DF_OUT "\n";
  for (my $i=0; $i <= 11; $i++) {
      print DF_OUT "$ds1[$i]";
# print "$ds1[$i]";
  }
  print DF_OUT "\n";
  for (my $i=0; $i <= 11; $i++) {
      print DF_OUT "$ds2[$i]";
  }

# <!-- Round Robin Archives -->   <rra>
# reading data points - 5 cycles (data and RRA) and till the end
# prepare RRA ds 12 lines
 @ds1 = ("                      <ds>
","                     <primary_value> NaN </primary_value>
","                     <secondary_value> NaN </secondary_value>
","                     <value> NaN </value>
","                     <unknown_datapoints> 0 </unknown_datapoints>
","                     </ds>
","                     <ds>
","                     <primary_value> NaN </primary_value>
","                     <secondary_value> NaN </secondary_value>
","                     <value> NaN </value>
","                     <unknown_datapoints> 0 </unknown_datapoints>
","                     </ds>
");

 for (my $cycle=0; $cycle <= 4; $cycle++) {
     while ( $line = shift @lines) {
        if ($line =~ "</v></row>") {
           $line =~ s/<\/row>/<v> NaN <\/v><v> NaN <\/v><\/row>/;
        }
        last if ( $line =~ "</cdp_prep>" );
        print DF_OUT "$line";
     }
# print next two ds
     for (my $i=0; $i <= 11; $i++) {
         print DF_OUT "$ds1[$i]";
     }
     print DF_OUT "$line";
     next;
 }    # end of for
 while ( $line = shift @lines) {
     if ($line =~ "</v></row>") {
        $line =~ s/<\/row>/<v> NaN <\/v><v> NaN <\/v><\/row>/;
     }
     print DF_OUT "$line";
 }
 close(DF_OUT) || error("Cannot close $f_dumped: $!".__FILE__.":".__LINE__) && return 0;

 (! `mv $rrd_file $rrd_file.backup`) || error("Cannot backup $rrd_file: $!".__FILE__.":".__LINE__) && return 0;  
 (! `rrdtool restore $f_dumped $rrd_file`) || error("Cannot restore $rrd_file: $!".__FILE__.":".__LINE__) && return 0; 
 unlink($f_dumped);
 unlink("$rrd_file.backup");

  # create lpar directory and file hard link into the other HMC if there is dual HMC setup

  my $found = 0;
  my $server_space = $server;
  if ( $server =~ m/ / ) {
    $server_space = "\"".$server."\""; # it must be here to support space with server names
  }
  my @files = <$wrkdir/$server_space/*>;

#  my @files = <$wrkdir/"$server"/*>;
  my $rrd_dir = "";
  foreach my $rrd_dir_tmp (@files) {
    chomp($rrd_dir_tmp);
    if ( -d $rrd_dir_tmp ) {
      $found = 1;
      $rrd_dir = $rrd_dir_tmp;
      last;
    }
  }
  if ( $found == 0 ) {
     error ("en part iops: Could not found a HMC in : $wrkdir/$server");
     return 0;
  }

  my $rrd_dir_base = basename($rrd_dir);
  foreach my $rrd_dir_new (@files) {
    chomp($rrd_dir_new);
    my $rrd_dir_new_base = basename($rrd_dir_new);
    if ( -d $rrd_dir_new && $rrd_dir_new_base !~ m/^$rrd_dir_base$/ ) {
       if ( ! -d "$rrd_dir_new/$lpar/" ) {
          print_it ("mkdir dual     : $rrd_dir_new/$lpar/") if $DEBUG ;
          mkdir("$rrd_dir_new/$lpar/") || error("Cannot mkdir $rrd_dir_new/$lpar/: $!".__FILE__.":".__LINE__) && return 0;
       }
      print_it (" hard link      : $rrd_file --> $rrd_dir_new/$lpar/$ent.mmm") if $DEBUG ;
      my $rrd_link_new = "$rrd_dir_new/$lpar/$ent.mmm";
      unlink("$rrd_dir_new/$lpar/$ent.mmm");       #for every case
      link ($rrd_file,"$rrd_dir_new/$lpar/$ent.mmm") || error("Cannot link $rrd_dir_new/$lpar/: $!".__FILE__.":".__LINE__) && return 0;;
      touch (""); 
    }
  }


return (1);

}

# finds lpar name per lpar_id in data/server/hmc/cpu.cfg
# at first find newest cpu.cfg

sub find_lpar_name
{
  my $server = shift;
  my $lpar_id = shift;

  my $found = 0;
  my $server_space = $server;
  if ( $server =~ m/ / ) {
    $server_space = "\"".$server."\""; # it must be here to support space with server names
  }
  my @files = <$wrkdir/$server_space/*/cpu.cfg>;

  my $cpu_file_time = 0;
  my $cpu_file = "";
  foreach my $line (@files) {
    chomp($line);
    if ( ! -f $line ) {
      next; # just to be sure
    }
    my $cpu_file_time_tmp = (stat("$line"))[9];
    if ( $cpu_file_time_tmp > $cpu_file_time ) {
      $cpu_file_time = $cpu_file_time_tmp;
      $cpu_file = $line;
    }
  }
  if ( $cpu_file eq '' ) {
     print_it ("LPAR ID not found:$server:$lpar_id no cpu.cfg") if $DEBUG > 2;
     return "";
  }
  open(FHC, "< $cpu_file") || error("Can't open $cpu_file : $!".__FILE__.":".__LINE__) && return "";
  my @res_all = <FHC>;
  close (FHC);
  foreach my $line (@res_all) {
    chomp ($line);
    if ( $line =~ m/,lpar_id=$lpar_id,/ ) {
      # lpar_id has been found
      $line =~ s/^lpar_name=//;
      $line =~ s/,lpar_id=.*//;
      print_it ("LPAR ID found   : $server:$line:$lpar_id") if $DEBUG > 2;
      return $line;
    }
  }

  print_it ("LPAR ID not found:$server:$lpar_id no line in cpu.cfg") if $DEBUG > 2;
  return "";
}

sub store_data_10     # returns 0 - fatal error, original $time as OK 
{
  my $data = shift;
  my $last_rec = shift;
  my $protocol_version = shift;
  my $peer_address = shift;
  my $en_last_rec = $last_rec;
  my $act_time = localtime();
my $DEBUG = 0;
  #example of $data
  #8233-E8B*53842P:BSRV22LPAR9:9:1392111120:2097152:2009792:87360:1243360:1962416:0:0:-1:4194304:135552:1143096:0:14824:85440:1836576:0:173216:619:31658:Tue Feb 11 10:32:00 2014:en2:192.168.201.69:1131735328:1493381247:en3:192.168.202.69:1099362694:1583796921:en4:172.31.216.179:25454855750:3192177654:fcs0:0xC050760329FC00C8:26690353045:90825841152:fcs0:iops:13678924:23553862:fcs1:0xC050760329FC00CA:15223131166:86896404992:fcs1:iops:1904252:15878939:tps:tps:4096:4


  my $server = "";
  my $lpar_name = "";
  my $time = "";
  my $size = "";
  my $inuse = "";
  my $free = "";
  my $pin = "";
  my $virtual = "";
  my $available = "";
  my $loaned = "";
  my $mmode = "";
  my $size_pg = "";
  my $inuse_pg = "";
  my $pin_work = "";
  my $pin_pers = "";
  my $pin_clnt = "";
  my $pin_other = "";
  my $in_use_work = "";
  my $in_use_pers = "";
  my $in_use_clnt = "";
  my $page_in = "";
  my $page_out = "";
  my $dat1 = "";
  my $dat2 = "";
  my $dat3 = "";
  my $en = "";
  my $enip = "";
  my $entransb = "";
  my $enrecb = "";
  my $dfc5 = "";
  my $dfc6 = "";
  my $dfc7 = "";
  my $dfc8 = "";
  my $lpar_id = -1;


  if ( $protocol_version < 11 ) {
    ($server, $lpar_name, $time, $size,$inuse,$free,$pin,$virtual,$available,$loaned,
    $mmode,$size_pg,$inuse_pg,$pin_work,$pin_pers,$pin_clnt,$pin_other,$in_use_work,
    $in_use_pers,$in_use_clnt,$page_in,$page_out,$dat1,$dat2,$dat3,
    $en,$enip, $entransb,$enrecb, $dfc5, $dfc6, $dfc7, $dfc8) = split (/:/,$data);
  }
  else {
    # lpar_id on the 3rd possition
    ($server, $lpar_name, $lpar_id, $time, $size,$inuse,$free,$pin,$virtual,$available,$loaned,
    $mmode,$size_pg,$inuse_pg,$pin_work,$pin_pers,$pin_clnt,$pin_other,$in_use_work,
    $in_use_pers,$in_use_clnt,$page_in,$page_out,$dat1,$dat2,$dat3,
    $en,$enip, $entransb,$enrecb, $dfc5, $dfc6, $dfc7, $dfc8) = split (/:/,$data);
    if ( $lpar_id > 1200000000 ) {
      # obviously still old data comming via a new protocol, just to be sure and do not case a gap after the agent upgrade
      ($server, $lpar_name, $time, $size,$inuse,$free,$pin,$virtual,$available,$loaned,
      $mmode,$size_pg,$inuse_pg,$pin_work,$pin_pers,$pin_clnt,$pin_other,$in_use_work,
      $in_use_pers,$in_use_clnt,$page_in,$page_out,$dat1,$dat2,$dat3,
      $en,$enip, $entransb,$enrecb, $dfc5, $dfc6, $dfc7, $dfc8) = split (/:/,$data);
    }
  }

  # Find out lpar name from lpar_id if it has not been done yet
  if ( $lpar eq '' && $lpar_id != -1 ) {
    my $lpar_name_id = find_lpar_name($server,$lpar_id);
    if ( ! $lpar_name_id eq '' ) {
      $lpar = $lpar_name_id;  # lpar name from lparstat -i does not have to be actual, linux on power does not provide lpar name at all
    }
  }
  if ( $lpar eq '' && ! $lpar_name eq '' ) {
    # just make sure if lpar-id fails somehow then use transferred $lpar_name
    $lpar = $lpar_name;
  }
  if ( $lpar eq '' ) {
    error("lpar name has not been found for client: $peer_address , server:$server, lpar_id:$lpar_id".__FILE__.":".__LINE__);
    return (0);
  }

  print "$data" if $DEBUG > 2;

  my $lpar_real = $lpar;
  $lpar_real =~ s/\//&&1/g;

  my $rrd_file = "";
  my $lpar_space = $lpar_real;
  if ( $lpar_real =~ m/ / ) {
    $lpar_space = "\"".$lpar_real."\""; # it must be here to support space with lpar names
  }
  my $server_space = $server;
  if ( $server =~ m/ / ) {
    $server_space = "\"".$server."\""; # it must be here to support space with server names
  }

  my @files = <$wrkdir/$server_space/*/$lpar_space/mem.mmm>;
  foreach my $rrd_file_tmp (@files) {
    chomp($rrd_file_tmp);
    $rrd_file = $rrd_file_tmp;
    last;
  }

  if ( ! $rrd_file eq '' ) {
    my $filesize = -s "$rrd_file";
    if ( $filesize == 0 ) {
      # when a FS is full then it creates 0 Bytes rrdtool files what is a problem, delete it then
      error("0 size rrd file: $rrd_file  - delete it".__FILE__.":".__LINE__);
      unlink ("$rrd_file") || error ("Cannot rm $rrd_file : $!");
      $rrd_file = ""; # force to create a new one
    }
  }

  print "$act_time: Updating       : $server:$lpar - $rrd_file\n" if $DEBUG > 2;
  if ( $rrd_file eq '' ) {
    if ( create2_rrd($server,$lpar_real,$time,$server_space,$lpar_space,"mem.mmm") == 0 ) {
    #if ( create_rrd($server,$lpar_real,$time) == 0 ) {
      return 0;
    }
    $rrd_file = "";
    @files = <$wrkdir/$server_space/*/$lpar_space/pgs.mmm>;
    foreach my $rrd_file_tmp (@files) {
       chomp($rrd_file_tmp);
       $rrd_file = $rrd_file_tmp;
       last;
    }
    if ( ! -f $rrd_file || $rrd_file eq "") { 
      if ( create2_rrd($server,$lpar_real,$time,$server_space,$lpar_space,"pgs.mmm") == 0 ) {
        return 0;
      }
    }
    @files = <$wrkdir/$server_space/*/$lpar_space/mem.mmm>;
    foreach my $rrd_file_tmp (@files) {
      chomp($rrd_file_tmp);
      $rrd_file = $rrd_file_tmp;
      last;
    }
  }

  if ($last_rec == 0 ) {
    # construction against crashing daemon Perl code when RRDTool error appears
    # this does not work well in old RRDTOool: $RRDp::error_mode = 'catch';
    # construction is not too costly as it runs once per each load
    eval {
      RRDp::cmd qq(last "$rrd_file" );
      my $last_rec_rrd = RRDp::read;
      chomp ($$last_rec_rrd);
      $last_rec = $$last_rec_rrd;
    };
    if ($@) {
      rrd_error($@.__FILE__.":".__LINE__,$rrd_file);
      return 0;
    }
  }


  if ( ($last_rec + $STEP/2) >= $time ) {
    #error("$server:$lpar : last rec : $last_rec + $STEP/2 >= $time, ignoring it ...".__FILE__.":".__LINE__);
    return $time; # it is also wrong, must be returned original time, not last_rec
	# --> no, no, it is not wrong, just ignore it!
  }

  #
  # Memory file update 
  #

  #print "000 rrd_file $rrd_file $time:$size:$inuse:$free:$pin:$in_use_work:$in_use_clnt\n";
  RRDp::cmd qq(update "$rrd_file" $time:$size:$inuse:$free:$pin:$in_use_work:$in_use_clnt);

  my $answer = RRDp::read;
  if ( ! $$answer eq '' && $$answer =~ m/ERROR/ ) {
    error (" $server:$lpar : $rrd_file : $time:$size:$inuse:$free:$pin:$virtual:$available ... : $$answer");
    if ( $$answer =~ m/is not an RRD file/ ) {
      (my $err,  my $file, my $txt) = split(/'/,$$answer);
      error ("Removing as it seems to be corrupted: $file");
      unlink("$file") || error ("Cannot rm $file : $!");
    }
    return 0;
  }

  #
  # paging data
  #

  $rrd_file =~ s/mem\.mmm/pgs\.mmm/g;
  #print "001 rrd_file $rrd_file $time:$page_in:$page_out:$size_pg:U\n";
  my $nan = "U";
  RRDp::cmd qq(update "$rrd_file" $time:$page_in:$page_out:$size_pg:$nan);
  
  $answer = RRDp::read;
  if ( ! $$answer eq '' && $$answer =~ m/ERROR/ ) {
    error (" $server:$lpar : $rrd_file : $time:$size:$inuse:$free:$pin:$virtual:$available ... : $$answer");
    if ( $$answer =~ m/is not an RRD file/ ) {
      (my $err,  my $file, my $txt) = split(/'/,$$answer);
      error ("Removing as it seems to be corrupted: $file");
      unlink("$file") || error ("Cannot rm $file : $!");
    }
    return 0;
  }

 print "finish storing data from agent\n" if $DEBUG > 2;
 return $time;             # return time of last record
}

sub create_en_rrd
{
  my $server = shift;
  my $lpar = shift;
  my $time = shift;
  my $ent = shift;
  my $no_time = $STEP * 7; # says the time interval when RRDTOOL consideres a gap in input data
  my $act_time = localtime();

  my $found = 0;
  my $rrd_dir = "";
  my $server_space = $server;
  if ( $server =~ m/ / ) {
    $server_space = "\"".$server."\""; # it must be here to support space with server names
  }
  my @files = <$wrkdir/$server_space/*>;
  foreach my $rrd_dir_tmp (@files) {
    chomp($rrd_dir_tmp);
    if ( -d $rrd_dir_tmp ) {
      $found = 1;
      $rrd_dir = $rrd_dir_tmp;
      last;
    }
  }

  if ( $found == 0 ) {
    error ("en: Could not found a HMC in : $wrkdir/$server");
    return 0;
  }
  if ( ! -d "$rrd_dir/$lpar/" ) {
     print_it ("mkdir          : $rrd_dir/$lpar/") if $DEBUG ;
     mkdir("$rrd_dir/$lpar/") || error("Cannot mkdir $rrd_dir/$lpar/: $!".__FILE__.":".__LINE__) && return 0;
     touch (""); 
  }

  my $rrd = "$rrd_dir/$lpar/$ent.mmm";
  print "en: $ent  $act_time: RRD create     : $rrd\n" if $DEBUG > 2;
  my $hb = 2 * $STEP;
  my $stepx = 15 * $STEP;  
  if ($ent eq "ame_mem") {
  RRDp::cmd qq(create "$rrd"  --start "$time-$stepx" --step "$STEP"
       "DS:ame_mem:GAUGE:$hb:0:U"
       "DS:ame_ratio:GAUGE:$hb:0:1000"
       "DS:ame_deficit:GAUGE:$hb:0:U"
       "RRA:AVERAGE:0.5:1:$one_minute_sample"
       "RRA:AVERAGE:0.5:5:$five_mins_sample"
       "RRA:AVERAGE:0.5:60:$one_hour_sample"
       "RRA:AVERAGE:0.5:300:$five_hours_sample"
       "RRA:AVERAGE:0.5:1440:$one_day_sample"
    );
  }
  elsif ($ent eq "tps") {
   RRDp::cmd qq(create "$rrd"  --start "$time-$stepx" --step "$STEP"
       "DS:percent:GAUGE:$hb:0:100"
       "DS:paging_space:GAUGE:$hb:0:U"
       "RRA:AVERAGE:0.5:1:$one_minute_sample"
       "RRA:AVERAGE:0.5:5:$five_mins_sample"
       "RRA:AVERAGE:0.5:60:$one_hour_sample"
       "RRA:AVERAGE:0.5:300:$five_hours_sample"
       "RRA:AVERAGE:0.5:1440:$one_day_sample"
    );
  }
   elsif ($ent =~ m/^fcs/ ) {
   RRDp::cmd qq(create "$rrd"  --start "$time-$stepx" --step "$STEP"
       "DS:recv_bytes:COUNTER:$hb:0:U"
       "DS:trans_bytes:COUNTER:$hb:0:U"
       "DS:iops_in:COUNTER:$hb:0:U"
       "DS:iops_out:COUNTER:$hb:0:U"
       "RRA:AVERAGE:0.5:1:$one_minute_sample"
       "RRA:AVERAGE:0.5:5:$five_mins_sample"
       "RRA:AVERAGE:0.5:60:$one_hour_sample"
       "RRA:AVERAGE:0.5:300:$five_hours_sample"
       "RRA:AVERAGE:0.5:1440:$one_day_sample"
    );
  }

  else  {    # here is enxx
  RRDp::cmd qq(create "$rrd"  --start "$time-$stepx" --step "$STEP"
       "DS:recv_bytes:COUNTER:$hb:0:U"
       "DS:trans_bytes:COUNTER:$hb:0:U"
       "RRA:AVERAGE:0.5:1:$one_minute_sample"
       "RRA:AVERAGE:0.5:5:$five_mins_sample"
       "RRA:AVERAGE:0.5:60:$one_hour_sample"
       "RRA:AVERAGE:0.5:300:$five_hours_sample"
       "RRA:AVERAGE:0.5:1440:$one_day_sample"
    );
  }

  my $answer = RRDp::read;

  if ( $$answer ) {
    error ("unable to create $rrd: answer: $$answer");
    return 0;
  }

  # create lpar directory and file hard link into the other HMC if there is dual HMC setup
  my $rrd_dir_base = basename($rrd_dir);
  foreach my $rrd_dir_new (@files) {
    chomp($rrd_dir_new);
    my $rrd_dir_new_base = basename($rrd_dir_new);
    if ( -d $rrd_dir_new && $rrd_dir_new_base !~ m/^$rrd_dir_base$/ ) {
       if ( ! -d "$rrd_dir_new/$lpar/" ) {
          print_it ("mkdir dual     : $rrd_dir_new/$lpar/") if $DEBUG ;
          mkdir("$rrd_dir_new/$lpar/") || error("Cannot mkdir $rrd_dir_new/$lpar/: $!".__FILE__.":".__LINE__) && return 0;
       }
      print_it ("hard link      : $rrd --> $rrd_dir_new/$lpar/$ent.mmm") if $DEBUG ;
      my $rrd_link_new = "$rrd_dir_new/$lpar/$ent.mmm";
      unlink("$rrd_dir_new/$lpar/$ent.mmm");   # for sure  
      link ($rrd,"$rrd_dir_new/$lpar/$ent.mmm") || error("Cannot link $rrd_dir_new/$lpar/: $!".__FILE__.":".__LINE__) && return 0;;
      touch (""); 
    }
  }

  return 1;
}

# stdout handling
sub print_it
{
  my $text = shift;
  my $act_time = localtime();
  chomp ($text);

  #print "ERROR          : $text : $!\n";
  print "$act_time: $text\n";

  return 1;
}


# error handling
sub error
{
  my $text = shift;
  my $act_time = localtime();
  chomp ($text);

  #print "ERROR          : $text : $!\n";
  print STDERR "$act_time: $text : $!\n";

  return 1;
}

sub load_retentions
{
  my $step=shift;
  my $basedir = shift;

  # standards
  $one_minute_sample=86400;
  $five_mins_sample=25920;
  $one_hour_sample=4320;
  $five_hours_sample=1734;
  $one_day_sample=1080;

  if ( ! -f "$basedir/etc/retention.cfg" ) {
    # standard retentions in place
    return 0;
  }

  # extra retentions are specifiled in $basedir/etc/retention.cfg
  open(FH, "< $basedir/etc/retention.cfg") || error ("Can't read from: $basedir/etc/retention.cfg: $!");

  my @lines = <FH>;
  foreach my $line (@lines) {
    chomp ($line);
    if ( $line =~ m/^1min/ ) {
      (my $trash, $one_minute_sample) = split(/:/,$line);
    }
    if ( $line =~ m/^5min/ ) {
      (my $trash, $five_mins_sample) = split(/:/,$line);
    }
    if ( $line =~ m/^60min/ ) {
      (my $trash, $one_hour_sample) = split(/:/,$line);
    }
    if ( $line =~ m/^300min/ ) {
      (my $trash, $five_hours_sample) = split(/:/,$line);
    }
    if ( $line =~ m/^1440min/ ) {
      (my $trash, $one_day_sample) = split(/:/,$line);
    }
  }

  close (FH);

  $step = $step / 60;
  $one_minute_sample = $one_minute_sample / $step;
  $five_mins_sample = $five_mins_sample / $step;
  $one_hour_sample = $one_hour_sample / $step;
  $five_hours_sample = $five_hours_sample / $step;
  $one_day_sample = $one_day_sample / $step;

  return 1;
}

sub basename {
  my $full = shift;
  my $out = "";

  # basename without direct function
  my @base = split(/\//,$full);
  foreach my $m (@base) {
    $out = $m;
  }

  return $out;
}

sub rrd_error
{
  my $err_text = shift;
  my $rrd_file = shift;
  my $tmpdir = "$basedir/tmp";
  if (defined $ENV{TMPDIR}) {
    $tmpdir = $ENV{TMPDIR};
  }

  chomp ($err_text);

  if ( $err_text =~ m/ERROR:/ && ! $rrd_file eq '' ) {
    # copy of the corrupted file into "save" place and remove the original one
    copy ("$rrd_file","$tmpdir/") || error("Cannot: cp $rrd_file $tmpdir/: $!".__FILE__.":".__LINE__);
    unlink("$rrd_file") || error("Cannot rm $rrd_file : $!".__FILE__.":".__LINE__);
    error("$err_text, moving it into: $tmpdir/");
  }
  else {
    error("$err_text");
  }
  return 0;
}

# extends fcsxx file for iops in-out

sub fcs_iops {
    my $rrd_file = shift;
    my $server = shift;
    my $lpar = shift;
    my $ent = shift;
#  read dumped file, defining new ds,
   my $f_dumped = "/tmp/rrd_dump";
   unlink($f_dumped); #in any case
   (! `rrdtool dump $rrd_file $f_dumped`) || error("Cannot dump $rrd_file to $f_dumped: $!".__FILE__.":".__LINE__) && return 0;
  open (DF_IN,"< $f_dumped") || error("Cannot open for reading $f_dumped: $!".__FILE__.":".__LINE__) && return 0;
  my @lines = <DF_IN>;
  close (DF_IN);
  unlink($f_dumped);
  $f_dumped = "/tmp/rrd_dump_q";
  open (DF_OUT, "> $f_dumped") || error("Cannot open for writing $f_dumped: $!".__FILE__.":".__LINE__) && return 0;

  my (@ds1, @ds2, $line );
  while ( $line = shift @lines) {
     print DF_OUT "$line";
     last if ( $line =~ "<ds>" );
  }
#print "$lines[0],$lines[1],$lines[2],$lines[3]\n";
  $ds1[0] = $line;
  for (my $i = 1; $i <= 11; $i++) {
      $line = shift @lines;
      print DF_OUT "$line";
      if ( $line =~ "recv_bytes" ) {
         $line =~ s/recv_bytes/iops_in/;
         $ds1[$i] = $line;
         next;
      }
      if ( $line =~ "<min>" ) {
         $ds1[$i] = "           <min> 0.0000000000e+00 </min>\n";
         next;
      }
      if ( $line =~ "<value>" ) {
         $ds1[$i] = "           <value> 0.0000000000e+00 </value>\n";
         next;
      }
      $ds1[$i] = $line;
      next;
  }
  while ( $line = shift @lines) {
     print DF_OUT "$line";
     last if ( $line =~ "<ds>" );
  }
  $ds2[0] = $line;
  for (my $i=1; $i <= 11; $i++) {
      $line = shift @lines;
      print DF_OUT "$line";
      if ( $line =~ "trans_bytes" ) {
         $line =~ s/trans_bytes/iops_out/;
         $ds2[$i] = $line;
         next;
      }
      if ( $line =~ "<min>" ) {
         $ds2[$i] = "           <min> 0.0000000000e+00 </min>\n";
         next;
      }
      if ( $line =~ "<value>" ) {
         $ds2[$i] = "           <value> 0.0000000000e+00 </value>\n";
         next;
      }
      $ds2[$i] = $line;
      next;
  }
# print next two ds
  print DF_OUT "\n";
  for (my $i=0; $i <= 11; $i++) {
      print DF_OUT "$ds1[$i]";
# print "$ds1[$i]";
  }
  print DF_OUT "\n";
  for (my $i=0; $i <= 11; $i++) {
      print DF_OUT "$ds2[$i]";
  }

# <!-- Round Robin Archives -->   <rra>
# reading data points - 5 cycles (data and RRA) and till the end
# prepare RRA ds 12 lines
 @ds1 = ("                      <ds>
","                     <primary_value> NaN </primary_value>
","                     <secondary_value> NaN </secondary_value>
","                     <value> NaN </value>
","                     <unknown_datapoints> 0 </unknown_datapoints>
","                     </ds>
","                     <ds>
","                     <primary_value> NaN </primary_value>
","                     <secondary_value> NaN </secondary_value>
","                     <value> NaN </value>
","                     <unknown_datapoints> 0 </unknown_datapoints>
","                     </ds>
");

 for (my $cycle=0; $cycle <= 4; $cycle++) {
     while ( $line = shift @lines) {
        if ($line =~ "</v></row>") {
           $line =~ s/<\/row>/<v> NaN <\/v><v> NaN <\/v><\/row>/;
        }
        last if ( $line =~ "</cdp_prep>" );
        print DF_OUT "$line";
     }
# print next two ds
     for (my $i=0; $i <= 11; $i++) {
         print DF_OUT "$ds1[$i]";
     }
     print DF_OUT "$line";
     next;
 }    # end of for
 while ( $line = shift @lines) {
     if ($line =~ "</v></row>") {
        $line =~ s/<\/row>/<v> NaN <\/v><v> NaN <\/v><\/row>/;
     }
     print DF_OUT "$line";
 }
 close(DF_OUT) || error("Cannot close $f_dumped: $!".__FILE__.":".__LINE__) && return 0;

 (! `mv $rrd_file $rrd_file.backup`) || error("Cannot backup $rrd_file: $!".__FILE__.":".__LINE__) && return 0;  
 (! `rrdtool restore $f_dumped $rrd_file`) || error("Cannot restore $rrd_file: $!".__FILE__.":".__LINE__) && return 0; 
 unlink($f_dumped);
 unlink("$rrd_file.backup");

  # create lpar directory and file hard link into the other HMC if there is dual HMC setup

  my $found = 0;
  my $server_space = $server;
  if ( $server =~ m/ / ) {
    $server_space = "\"".$server."\""; # it must be here to support space with server names
  }
  my @files = <$wrkdir/$server_space/*>;

#  my @files = <$wrkdir/"$server"/*>;
  my $rrd_dir = "";
  foreach my $rrd_dir_tmp (@files) {
    chomp($rrd_dir_tmp);
    if ( -d $rrd_dir_tmp ) {
      $found = 1;
      $rrd_dir = $rrd_dir_tmp;
      last;
    }
  }
  if ( $found == 0 ) {
     error ("en part iops: Could not found a HMC in : $wrkdir/$server");
     return 0;
  }

  my $rrd_dir_base = basename($rrd_dir);
  foreach my $rrd_dir_new (@files) {
    chomp($rrd_dir_new);
    my $rrd_dir_new_base = basename($rrd_dir_new);
    if ( -d $rrd_dir_new && $rrd_dir_new_base !~ m/^$rrd_dir_base$/ ) {
       if ( ! -d "$rrd_dir_new/$lpar/" ) {
          print_it ("mkdir dual     : $rrd_dir_new/$lpar/") if $DEBUG ;
          mkdir("$rrd_dir_new/$lpar/") || error("Cannot mkdir $rrd_dir_new/$lpar/: $!".__FILE__.":".__LINE__) && return 0;
       }
      print_it ("hard link      : $rrd_file --> $rrd_dir_new/$lpar/$ent.mmm") if $DEBUG ;
      my $rrd_link_new = "$rrd_dir_new/$lpar/$ent.mmm";
      unlink("$rrd_dir_new/$lpar/$ent.mmm");       #for every case
      link ($rrd_file,"$rrd_dir_new/$lpar/$ent.mmm") || error("Cannot link $rrd_dir_new/$lpar/: $!".__FILE__.":".__LINE__) && return 0;;
      touch (""); 
    }
  }


return (1);

}

# finds lpar name per lpar_id in data/server/hmc/cpu.cfg
# at first find newest cpu.cfg

sub find_lpar_name
{
  my $server = shift;
  my $lpar_id = shift;

  my $found = 0;
  my $server_space = $server;
  if ( $server =~ m/ / ) {
    $server_space = "\"".$server."\""; # it must be here to support space with server names
  }
  my @files = <$wrkdir/$server_space/*/cpu.cfg>;

  my $cpu_file_time = 0;
  my $cpu_file = "";
  foreach my $line (@files) {
    chomp($line);
    if ( ! -f $line ) {
      next; # just to be sure
    }
    my $cpu_file_time_tmp = (stat("$line"))[9];
    if ( $cpu_file_time_tmp > $cpu_file_time ) {
      $cpu_file_time = $cpu_file_time_tmp;
      $cpu_file = $line;
    }
  }
  if ( $cpu_file eq '' ) {
     print "LPAR ID not found:$server:$lpar_id no cpu.cfg\n" if $DEBUG > 2;
     return "";
  }
  open(FHC, "< $cpu_file") || error("Can't open $cpu_file : $!".__FILE__.":".__LINE__) && return "";
  my @res_all = <FHC>;
  close (FHC);
  foreach my $line (@res_all) {
    chomp ($line);
    if ( $line =~ m/,lpar_id=$lpar_id,/ ) {
      # lpar_id has been found
      $line =~ s/^lpar_name=//;
      $line =~ s/,lpar_id=.*//;
      print "LPAR ID found   : $server:$line:$lpar_id\n" if $DEBUG > 2;
      return $line;
    }
  }

  print "LPAR ID not found:$server:$lpar_id no line in cpu.cfg\n" if $DEBUG > 2;
  return "";
}

sub isdigit
{
  my $digit = shift;
  my $text = shift;

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;

  if (length($digit_work) == 0) {
    # is a number
    return 1;
  }

  #if (($digit * 1) eq $digit){
  #  # is a number
  #  return 1;
  #}

  # NOT a number
  return 0;
}

sub touch
{
  my $text = shift;

  my $version="$ENV{version}";
  my $basedir = $ENV{INPUTDIR};
  my $new_change="$basedir/tmp/$version-run";
  my $host = $ENV{HMC};
  my $DEBUG = $ENV{DEBUG};


  if ( ! -f $new_change ) {
   `touch $new_change`; # say install_html.sh that there was any change
   #if ( $text eq '' ) {
   #  print "touch          : $host $new_change\n" if $DEBUG ;
   #}
   #else {
   #  print "touch          : $host $new_change : $text\n" if $DEBUG ;
   #}
  }

  return 0
}



# reduces file 'lpar.mmm' into 'lpar/mem.mmm'
# at the same time creates 'lpar/pgs.mm'
# reduction:
 # leave out data streams X
 # data streams O export to new pgs.mmm
 # 
 #  RRDp::cmd qq(create "$rrd"  --start "$time"  --step "$STEP"
 #   "DS:size:GAUGE:$no_time:0:102400000"
 #   "DS:nuse:GAUGE:$no_time:0:102400000"
 #   "DS:free:GAUGE:$no_time:0:102400000"
 #   "DS:pin:GAUGE:$no_time:0:102400000"
 # X  "DS:virtual:GAUGE:$no_time:0:102400000"
 # X  "DS:available:GAUGE:$no_time:0:102400000"
 # X  "DS:loaned:GAUGE:$no_time:0:102400000"
 # X  "DS:mmode:GAUGE:$no_time:0:20"
 # X  "DS:size_pg:GAUGE:$no_time:0:102400000"
 # X  "DS:inuse_pg:GAUGE:$no_time:0:102400000"
 # X  "DS:pin_work:GAUGE:$no_time:0:102400000"
 # X  "DS:pin_pers:GAUGE:$no_time:0:102400000"
 # X  "DS:pin_clnt:GAUGE:$no_time:0:102400000"
 # X  "DS:pin_other:GAUGE:$no_time:0:102400000"
 #   "DS:in_use_work:GAUGE:$no_time:0:102400000"
 # X  "DS:in_use_pers:GAUGE:$no_time:0:102400000"
 #   "DS:in_use_clnt:GAUGE:$no_time:0:102400000"
 # O  "DS:page_in:COUNTER:$no_time:0:U"
 # O  "DS:page_out:COUNTER:$no_time:0:U"
 #
 #  retentions are taken from original file 'lpar.mmm'

 #  creating pgs.mmm
 #
 #   RRDp::cmd qq(create "$rrd"  --start "$time" --step "$STEP"
 #    "DS:page_in:COUNTER:$no_time:0:U"
 #    "DS:page_out:COUNTER:$no_time:0:U"
 #    "DS:paging_space:GAUGE:$no_time:0:U"
 #    "DS:percent:GAUGE:$no_time:0:100"
 #
 #  retentions are taken from original file 'lpar.mmm'
 #
 #  algorithm:
 #  -  if workdir/servers/hmcs/'lpar'.mmm ds names are as above ? no -> ret0
 #  -  if not exists dir workdir/servers/hmcs/lpar/ then create
 #  -  convert from 'lpar.mmm' -> lpar/mem.mmm and lpar/pgs.mmm 
 #  -  create hard links if dual hmc setup
 #   -------------- 

 # original data 'lpar.mmm'
 
 #<!-- Round Robin Database Dump --><rrd> <version> 0003 </version>
 #       <step> 60 </step> <!-- Seconds -->
 #       <lastupdate> 1392645960 </lastupdate> <!-- 2014-02-17 15:06:00 GMT+01:00 -->
 #
 #       <ds>
 #               <name> size </name>
 #               <type> GAUGE </type>
 #               <minimal_heartbeat> 120 </minimal_heartbeat>
 #               <min> 0.0000000000e+00 </min>
 #               <max> 1.0240000000e+08 </max>
 #
 #               <!-- PDP Status -->
 #               <last_ds> UNKN </last_ds>
 #               <value> 0.0000000000e+00 </value>
 #               <unknown_sec> 0 </unknown_sec>
 #       </ds>
 #
 #       <ds>
 #               <name> nuse </name>
 # and so on
 #                <unknown_sec> 0 </unknown_sec>
 #       </ds>

 #<!-- Round Robin Archives -->   <rra>
 #               <cf> AVERAGE </cf>
 #               <pdp_per_row> 1 </pdp_per_row> <!-- 60 seconds -->

 #               <params>
 #               <xff> 5.0000000000e-01 </xff>
 #               </params>
 #               <cdp_prep>
 #                       <ds>
 #                       <primary_value> 1.0485760000e+06 </primary_value>
 #                       <secondary_value> NaN </secondary_value>
 #                       <value> NaN </value>
 #                       <unknown_datapoints> 0 </unknown_datapoints>
 #                       </ds>
 #                       <ds>
 #                       <primary_value> 1.0402800000e+06 </primary_value>
 # and so on
 #                         <unknown_datapoints> 0 </unknown_datapoints>
 #                       </ds>
 #                       <ds>
 #                       <primary_value> 0.0000000000e+00 </primary_value>
 #                       <secondary_value> NaN </secondary_value>
 #                       <value> NaN </value>
 #                       <unknown_datapoints> 0 </unknown_datapoints>
 #                       </ds>
 #               </cdp_prep>
 #               <database>
 #                       <!-- 2013-12-22 08:45:00 CET / 1387698300 --> <row><v> 1.0485760000e+06 </v><v> 1.0434915333e+06 </v><v> 5.0805333333e+03 </v><v> 4.0328000000e+05 </v><v> 6.2166573333e+05 </v><v> 3.7554233333e+05 </v><v> 0.0000000000e+00 </v><v> 0.0000000000e+00 </v><v> 3.2768000000e+06 </v><v> 6.5034000000e+03 </v><v> 3.5370000000e+05 </v><v> 0.0000000000e+00 </v><v> 0.0000000000e+00 </v><v> 4.9580000000e+04 </v><v> 6.2166573333e+05 </v><v> 0.0000000000e+00 </v><v> 4.2182580000e+05 </v><v> 0.0000000000e+00 </v><v> 0.0000000000e+00 </v></row>
 #                       <!-- 2013-12-22 08:46:00 CET / 1387698360 --> <row><v> 1.0485760000e+06 </v><v> 1.0446240000e+06 < 
 # and so on
 #                        <!-- 2014-02-20 08:44:00 CET / 1392882240 --> <row><v> 1.0485760000e+06 </v><v> 1.0402800000e+06 </v><v> 7.7720000000e+03 </v><v> 4.1174000000e+05 </v><v> 6.3619600000e+05 </v><v> 3.6001200000e+05 </v><v> 0.0000000000e+00 </v><v> 0.0000000000e+00 </v><v> 3.2768000000e+06 </v><v> 6.5280000000e+03 </v><v> 3.6216000000e+05 </v><v> 0.0000000000e+00 </v><v> 0.0000000000e+00 </v><v> 4.9580000000e+04 </v><v> 6.3567200000e+05 </v><v> 0.0000000000e+00 </v><v> 4.0460800000e+05 </v><v> 0.0000000000e+00 </v><v> 0.0000000000e+00 </v></row>
 #                </database>
 #       </rra>
 #       <rra>
 #               <cf> AVERAGE </cf>
 #               <pdp_per_row> 5 </pdp_per_row> <!-- 300 seconds -->

 #                <params>
 #               <xff> 5.0000000000e-01 </xff>
 #               </params>
 #               <cdp_prep>
 #                       <ds>
 #                       <primary_value> 1.0485760000e+06 </primary_value>
 # and so on
 #
 #              <pdp_per_row> 60 </pdp_per_row> <!-- 3600 seconds -->
 #              <pdp_per_row> 300 </pdp_per_row> <!-- 18000 seconds --> 
 #              <pdp_per_row> 1440 </pdp_per_row> <!-- 86400 seconds -->

 #

 #  ****    main  ****
 #

sub conv_lpar 
{ 
 my $rrd_file = shift;  # file to convert, usually "wrkdir/server/hmc/lpar.mmm"
 my $server = shift;
 my $lpar = shift;
 my $rrd_dir = shift; # "wrkdir/server/hmc/lpar"
 my $tmpd = shift;

# flush after every write
  $| = 1;
  my @filestttt = glob "$tmpd/*.tttt";
  if ( $filestttt[0] ne "" ) {
    `rm -r $tmpd/*.tttt`;
  }

   # start RRD pipe
  # RRDp::start "$rrdtool";
 my $tmpdir = "$tmpd/$server.$lpar.tttt";
      mkdir("$tmpdir") || error("Cannot mkdir $tmpdir: $!".__FILE__.":".__LINE__);
      my $f_dumped = "$tmpdir/rrd_dump";
      unlink($f_dumped);                    #in any case

      RRDp::cmd qq(dump "$rrd_file" "$f_dumped");  
      my $answer = RRDp::read;
      if ( $$answer =~ "ERROR" ) {
         error (" Convert rrdtool error : $$answer");
         if ( $$answer =~ "is not an RRD file" ) {
            (my $err,  my $file, my $txt) = split(/'/,$$answer);
            error ("It needs to be removed due to corruption: $file");
         }
         else {
            error ("Convert rrdtool error : $$answer");
         }
         return 0;
      }

      open (DF_IN,"< $f_dumped") || error("Cannot open for reading $f_dumped: $!".__FILE__.":".__LINE__) && return 0;

      my $f_pgs = "$tmpdir/rrd_dump_pgs";
      open (DF_OUT, "> $f_pgs") || error("Cannot open for writing $f_pgs: $!".__FILE__.":".__LINE__) && return 0;

      my $line ;
      while ( $line = <DF_IN>) {  # beginning
           print DF_OUT "$line";
           last if ( $line =~ "<rrd>" );
      }
      my $version_dump = 0 ; # our version
      if ($line !~ "version") {
         $version_dump = 1 ;    # FOUCOU version
      } 
      while ( $line = <DF_IN>) {  # beginning
           print DF_OUT "$line";
           last if ( $line =~ "<ds>" );
      }
      while ( $line = <DF_IN>) {
           last if ( $line =~ "<name> page_in </name>" ); # both version
      }
      print DF_OUT "$line";
      while ( $line = <DF_IN>) {      #R&W page_in page_out
           print DF_OUT "$line";
           last if ( $line =~ "<ds>" );
      }
      while ( $line = <DF_IN>) {
           print DF_OUT "$line";
           last if ( $line =~ "</ds>" );
      }
      if ($version_dump == 0 ) {
        print DF_OUT "
	<ds>
		<name> paging_space </name>
		<type> GAUGE </type>
		<minimal_heartbeat> 120 </minimal_heartbeat>
		<min> 0.0000000000e+00 </min>
		<max> NaN </max>

		<!-- PDP Status -->
		<last_ds> UNKN </last_ds>
		<value> 0.0000000000e+00 </value>
		<unknown_sec> 0 </unknown_sec>
	</ds>

	<ds>
		<name> percent </name>
		<type> GAUGE </type>
		<minimal_heartbeat> 120 </minimal_heartbeat>
		<min> 0.0000000000e+00 </min>
		<max> 1.0000000000e+02 </max>

		<!-- PDP Status -->
		<last_ds> UNKN </last_ds>
		<value> 0.0000000000e+00 </value>
		<unknown_sec> 0 </unknown_sec>
	</ds>
";
       }
       else {     # version FOUCOU
        print DF_OUT "
	<ds>
		<name> paging_space </name>
		<type> GAUGE </type>
		<minimal_heartbeat>120</minimal_heartbeat>
		<min>0.0000000000e+00</min>
		<max>NaN</max>

		<!-- PDP Status -->
		<last_ds>UNKN</last_ds>
		<value>0.0000000000e+00</value>
		<unknown_sec> 0 </unknown_sec>
	</ds>

	<ds>
		<name> percent </name>
		<type> GAUGE </type>
		<minimal_heartbeat>120</minimal_heartbeat>
		<min>0.0000000000e+00</min>
		<max>1.0000000000e+02</max>

		<!-- PDP Status -->
		<last_ds>UNKN</last_ds>
		<value>0.0000000000e+00</value>
		<unknown_sec> 0 </unknown_sec>
	</ds>
";
   }
# <!-- Round Robin Archives -->   <rra>
# reading data points - 5 cycles (RRA definition and data points) and till the end 

    my $rra_lines;
    if ( $version_dump == 0 ) {
        $rra_lines = "			<ds>
			<primary_value> NaN </primary_value>
			<secondary_value> NaN </secondary_value>
			<value> NaN </value>
			<unknown_datapoints> 0 </unknown_datapoints>
			</ds>
			<ds>
			<primary_value> NaN </primary_value>
			<secondary_value> NaN </secondary_value>
			<value> NaN </value>
			<unknown_datapoints> 0 </unknown_datapoints>
			</ds>
";
     }
    else {
        $rra_lines = "			<ds>
			<primary_value>NaN</primary_value>
			<secondary_value>NaN</secondary_value>
			<value>NaN</value>
			<unknown_datapoints>0</unknown_datapoints>
			</ds>
			<ds>
			<primary_value>NaN</primary_value>
			<secondary_value>NaN</secondary_value>
			<value>NaN</value>
			<unknown_datapoints>0</unknown_datapoints>
			</ds>
";
   }

    for (my $cycle=0; $cycle <= 4; $cycle++) {
        while ( $line = <DF_IN>) {      # until ds
              print DF_OUT "$line";
              last if ( $line =~ "<ds>" );
        }
        for (my $ix = 1; $ix < 18; $ix++ ) {
           while ( $line = <DF_IN>) {      # leave out until page_in ds
                 last if ( $line =~ "<ds>" );
           }
        }
        while ( $line = <DF_IN>) {      # for page_in 
              print DF_OUT "$line";
              last if ( $line =~ "<ds>" );
        }
        while ( $line = <DF_IN>) {      # for page_out_
              print DF_OUT "$line";
              last if ( $line =~ "</ds>" );
        }
        print DF_OUT "$rra_lines";             # and next two ds    
        $line = <DF_IN>;
        print DF_OUT "$line";
  
        while ( $line = <DF_IN>) {
           if ($line =~ /<\/v><\/row>/) {
              (my $p1, undef, undef, undef, undef, undef,
                       undef, undef, undef, undef, undef,
                       undef, undef, undef, undef, undef,
                       undef, undef, my $p2, my $p3) = split (/<v>/,$line );
              if ( $version_dump == 0 ) {
                 $p3 =~ s/<\/row>/<v> NaN <\/v><v> NaN <\/v><\/row>/;
              }
              else {
                 $p3 =~ s/<\/row>/<v>NaN<\/v><v>NaN<\/v><\/row>/; 
              }  
              $line = $p1."<v>".$p2."<v>".$p3;
           }
           print DF_OUT "$line";
           last if ( $line =~ "<cdp_prep>" );
        }
    }    # end of for cycle
 
    while ( $line = <DF_IN>) {      # until end of file 
            print DF_OUT "$line";
    }   # end of file 

   close(DF_OUT) || error("Cannot close $f_pgs: $!".__FILE__.":".__LINE__) && return 0;
   close(DF_IN)  || error("Cannot close $f_dumped: $!".__FILE__.":".__LINE__) && return 0;

#   2nd part: read dumped 'lpar.mmm', create mem.mmm,
   
   open (DF_IN,"< $f_dumped") || error("Cannot open for reading $f_dumped: $!".__FILE__.":".__LINE__) && return 0;
   my $f_mem = "$tmpdir/rrd_dump_mem";
   open (DF_OUT, "> $f_mem") || error("Cannot open for writing $f_mem: $!".__FILE__.":".__LINE__) && return 0;

      while ( $line = <DF_IN>) {  # beginning
           print DF_OUT "$line";
           last if ( $line =~ "<ds>" );
      }
      while ( $line = <DF_IN>) {      # size
           print DF_OUT "$line";
           last if ( $line =~ "<ds>" );
      }
      while ( $line = <DF_IN>) {      # nuse
           print DF_OUT "$line";
           last if ( $line =~ "<ds>" );
      }
      while ( $line = <DF_IN>) {      # free
           print DF_OUT "$line";
           last if ( $line =~ "<ds>" );
      }
      while ( $line = <DF_IN>) {      # pin 
           print DF_OUT "$line";
           last if ( $line =~ "<ds>" );
      }
      while ( $line = <DF_IN>) {
           last if ( $line =~ "<name> in_use_work </name>" );
      }
      print DF_OUT "$line";
      while ( $line = <DF_IN>) {      # in_use_work
           print DF_OUT "$line";
           last if ( $line =~ "<ds>" );
      }
      while ( $line = <DF_IN>) {
           last if ( $line =~ "<name> in_use_clnt </name>" );
      }
      print DF_OUT "$line";
      while ( $line = <DF_IN>) {      # in_use_clnt_
           print DF_OUT "$line";
           last if ( $line =~ "</ds>" );
      }
      while ( $line = <DF_IN>) {
           last if ( $line =~ "Round Robin" );
      }
      print DF_OUT "\n$line";

  for (my $cycle=0; $cycle <= 4; $cycle++) {

      while ( $line = <DF_IN>) {      # until ds
           print DF_OUT "$line";
           last if ( $line =~ "<ds>" );
      }
      while ( $line = <DF_IN>) {      # size
           print DF_OUT "$line";
           last if ( $line =~ "<ds>" );
      }
      while ( $line = <DF_IN>) {      # nuse
           print DF_OUT "$line";
           last if ( $line =~ "<ds>" );
      }
      while ( $line = <DF_IN>) {      # free
           print DF_OUT "$line";
           last if ( $line =~ "<ds>" );
      }
      while ( $line = <DF_IN>) {      # pin
           print DF_OUT "$line";
           last if ( $line =~ "<ds>" );
      }
      for (my $ix = 1; $ix < 11; $ix++ ) {
         while ( $line = <DF_IN>) {      # leave out until in_use_work 
               last if ( $line =~ "<ds>" );
         }
      }
      while ( $line = <DF_IN>) {      # in_use_work
           print DF_OUT "$line";
           last if ( $line =~ "<ds>" );
      }
      while ( $line = <DF_IN>) {      # leave out until in_use_clnt
              last if ( $line =~ "<ds>" );
        }
      while ( $line = <DF_IN>) {      # in_use_clnt
           print DF_OUT "$line";
           last if ( $line =~ "</ds>" );
      }
      while ( $line = <DF_IN>) {      # leave out until in_use_clnt
              last if ( $line =~ "</cdp_prep>" );
        }
      print DF_OUT "$line";
      while ( $line = <DF_IN>) {
         if ($line =~ /<\/v><\/row>/) {
            (my $p1, my $size, my $nuse, my $free, my $pin, undef,
                    undef, undef, undef, undef, undef,
                    undef, undef, undef, undef, my $in_use_work,
                    undef, my $in_use_clnt, undef, undef) = split (/<v>/,$line );
            $line = $p1."<v>".$size."<v>".$nuse."<v>".$free."<v>".$pin."<v>".$in_use_work."<v>".$in_use_clnt."<\/row>\n";
        }
        print DF_OUT "$line";
        last if ( $line =~ "<cdp_prep>" );
      }
   }    # end of for cycle

   while ( $line = <DF_IN>) {      # until end of file
       print DF_OUT "$line";
   }    # end of file 

   close(DF_OUT) || error("Cannot close $f_mem: $!".__FILE__.":".__LINE__) && return 0;
   close(DF_IN)  || error("Cannot close $f_dumped: $!".__FILE__.":".__LINE__) && return 0;

   # test file pgs
   
    my $tmp_pgs = "$tmpdir/rrd_pgs";
    unlink("$tmp_pgs");
 
    RRDp::cmd qq(restore "$f_pgs" "$tmp_pgs");
    $answer = RRDp::read;
    if ( $$answer =~ "ERROR" ) {
       error (" Convert rrdtool error : $$answer");
       if ( $$answer =~ "is not an RRD file" ) {
          (my $err,  my $file, my $txt) = split(/'/,$$answer);
          error ("It needs to be removed due to corruption: $file");
       }
       else {
          error ("Convert rrdtool error : $$answer");
       }
       return 0;
    }

    $rrd_file = $tmp_pgs;
    $f_dumped = "$tmpdir/rrd_dump_pgs2";
    unlink($f_dumped);                    #in any case

    RRDp::cmd qq(dump "$rrd_file" "$f_dumped");
     $answer = RRDp::read;
     if ( $$answer =~ "ERROR" ) {
        error (" Convert rrdtool error : $$answer");
        if ( $$answer =~ "is not an RRD file" ) {
           (my $err,  my $file, my $txt) = split(/'/,$$answer);
           error ("It needs to be removed due to corruption: $file");
        }
        else {
           error ("Convert rrdtool error : $$answer");
        }
        return 0;
     }
     my $f_one = "$tmpdir/rrd_dump_pgs";
     my $f_two = "$tmpdir/rrd_dump_pgs2";
     if (compare("$f_one","$f_two") != 0) {
        error("$f_one, $f_two are not the same for $rrd_dir : $!".__FILE__.":".__LINE__) && return 0;
     }
     unlink("$f_one");
     unlink("$f_two");
   
   # test file mem

    my $tmp_mem = "$tmpdir/rrd_mem";
    unlink("$tmp_mem");

    RRDp::cmd qq(restore "$f_mem" "$tmp_mem");
    $answer = RRDp::read;
    if ( $$answer =~ "ERROR" ) {
       error (" Convert rrdtool error : $$answer");
       if ( $$answer =~ "is not an RRD file" ) {
          (my $err,  my $file, my $txt) = split(/'/,$$answer);
          error ("It needs to be removed due to corruption: $file");
       }
       else {
          error ("Convert rrdtool error : $$answer");
       }
       return 0;
    }

    $rrd_file = $tmp_mem;
    $f_dumped = "$tmpdir/rrd_dump_mem2";
    unlink($f_dumped);                    #in any case

    RRDp::cmd qq(dump "$rrd_file" "$f_dumped");
     $answer = RRDp::read;
     if ( $$answer =~ "ERROR" ) {
        error (" Convert rrdtool error : $$answer");
        if ( $$answer =~ "is not an RRD file" ) {
           (my $err,  my $file, my $txt) = split(/'/,$$answer);
           error ("It needs to be removed due to corruption: $file");
        }
        else {
           error ("Convert rrdtool error : $$answer");
        }
        return 0;
     }
     $f_one = "$tmpdir/rrd_dump_mem";
     $f_two = "$tmpdir/rrd_dump_mem2";
     if (compare("$f_one","$f_two") != 0) {
        error("$f_one, $f_two are not the same for $rrd_dir : $!".__FILE__.":".__LINE__) && return 0;
     }
     unlink("$f_one");
     unlink("$f_two");

   # ready to place files in dir lpar/
   
     if ( ! -d "$rrd_dir/" ) {   
        mkdir("$rrd_dir/") || error("Cannot mkdir $rrd_dir/: $!".__FILE__.":".__LINE__) && return 0;
     }
     my $rrd_file_pgs = "$rrd_dir/pgs.mmm";
     my $rrd_file_mem = "$rrd_dir/mem.mmm";
     move("$tmpdir/rrd_pgs","$rrd_file_pgs") || error("Cannot move $rrd_dir/pgs.mmm: $!".__FILE__.":".__LINE__) && return 0;     
     move("$tmpdir/rrd_mem","$rrd_file_mem") || error("Cannot move $rrd_dir/pgs.mmm: $!".__FILE__.":".__LINE__) && return 0;
   
     unlink("$tmpdir/rrd_dump");

  # create lpar directory and file hard link into the other HMC if there is dual HMC setup

     my $found = 0;
     my $server_space = $server;
     if ( $server =~ m/ / ) {
       $server_space = "\"".$server."\""; # it must be here to support space with server names
     }
     my @files = <$wrkdir/$server_space/*>;

     $rrd_dir = "";
     foreach my $rrd_dir_tmp (@files) {
       chomp($rrd_dir_tmp);
       if ( -d $rrd_dir_tmp ) {
         $found = 1;
         $rrd_dir = $rrd_dir_tmp;
         last;
       }
     }
     if ( $found == 0 ) {
       error ("Convert: Could not found a HMC in : $wrkdir/$server $!".__FILE__.":".__LINE__) && return 0;
     }

     my $rrd_dir_base = basename($rrd_dir);
     foreach my $rrd_dir_new (@files) {
       chomp($rrd_dir_new);
       my $rrd_dir_new_base = basename($rrd_dir_new);
       if ( -d $rrd_dir_new && $rrd_dir_new_base !~ m/^$rrd_dir_base$/ ) {
          if ( ! -d "$rrd_dir_new/$lpar/" ) {
             print_it ("mkdir dual     : $rrd_dir_new/$lpar/") if $DEBUG ;
             mkdir("$rrd_dir_new/$lpar/") || error("Cannot mkdir $rrd_dir_new/$lpar/: $!".__FILE__.":".__LINE__) && return 0;
          }
          print_it ("hard link      : $rrd_file_mem --> $rrd_dir_new/$lpar/mem.mmm") if $DEBUG ;
          my $rrd_link_new = "$rrd_dir_new/$lpar/mem.mmm";
          unlink("$rrd_dir_new/$lpar/mem.mmm");       #for every case
          link ($rrd_file_mem,"$rrd_dir_new/$lpar/mem.mmm") || error("Cannot link $rrd_dir_new/$lpar/mem.mmm: $!".__FILE__.":".__LINE__) && return 0;

          print_it ("hard link      : $rrd_file_pgs --> $rrd_dir_new/$lpar/pgs.mmm") if $DEBUG ;
          $rrd_link_new = "$rrd_dir_new/$lpar/pgs.mmm";
          unlink("$rrd_dir_new/$lpar/pgs.mmm");       #for every case
          link ($rrd_file_pgs,"$rrd_dir_new/$lpar/pgs.mmm") || error("Cannot link $rrd_dir_new/$lpar/pgs.mmm: $!".__FILE__.":".__LINE__) && return 0;
          touch (""); 
       }
     }

     #  unlink the original 'lpar.mmm', care if double hmc
     foreach my $rrd_dir_new (@files) {
       chomp($rrd_dir_new); 
       if (-f "$rrd_dir_new/$lpar.mmm") {
          if (-f "$rrd_dir_new/$lpar/mem.mmm" && -f "$rrd_dir_new/$lpar/pgs.mmm") {
            print_it ("deleting org   : $rrd_dir_new/$lpar.mmm"); 
            unlink("$rrd_dir_new/$lpar.mmm");
          }
          else {
            print_it ("not delete org : conversion failed for: $rrd_dir_new/$lpar.mmm, do not exist: $rrd_dir_new/$lpar/mem.mmm - $rrd_dir_new/$lpar/pgs.mmm"); 
            error ("not delete org : conversion failed for: $rrd_dir_new/$lpar.mmm, do not exist: $rrd_dir_new/$lpar/mem.mmm - $rrd_dir_new/$lpar/pgs.mmm"); 
          }
       }
     }
     rmdir("$tmpdir") || error("Cannot rmdir $tmpdir: $!".__FILE__.":".__LINE__) ;

# close RRD pipe
#  RRDp::end;

  return 1;
}

sub  save_cpu_ghz 
{
  my $cpu_hz_global = shift;
  my $cpu_hz = shift;
  my $server = shift;
  my $wrkdir = shift;
  my $cpu_file = "cpu_mhz.txt";


  #print "001 $cpu_hz_global : $cpu_hz \n";
  if (isdigit($cpu_hz_global) && $cpu_hz_global > 1000000000) {
    return $cpu_hz_global; # already stored, skip it
  }
  
  if (!isdigit($cpu_hz) || $cpu_hz < 1000000000) {
    return 0; # some problem, ignore
  }

  my $cpu_mhz = $cpu_hz / 1000000;

  my $server_space = $server;
  if ( $server =~ m/ / ) {
    $server_space = "\"".$server."\""; # it must be here to support space with server names
  }

  #print "002 $cpu_hz_global : $cpu_hz : \n";
  my @hmcs  = <$wrkdir/$server_space/*>;
  foreach my $hmc (@hmcs) {
    chomp($hmc);
    if ( ! -d "$hmc" ) {
      next;
    }
  
    # update only once a day
    if ( -f "$hmc/$cpu_file" ) {
      #print "003 $cpu_hz_global : $cpu_hz : $hmc/$cpu_file\n";
      my $time = time();
      my $timem = ((stat("$hmc/$cpu_file"))[9]);
      if (( $time - $timem) < 86400 ){
        #print "004 $cpu_hz_global : $cpu_hz : $hmc/$cpu_file\n";
        return $cpu_hz; # no update now
      }
    }

    open(FH, "> $hmc/$cpu_file") || error("Can't open $hmc/$cpu_file.txt : $!".__FILE__.":".__LINE__) && return 0;
    print FH "$cpu_mhz\n";
    close (FH);
    print "Saving CPU MHZ into $hmc/$cpu_file : $cpu_mhz\n" if $DEBUG > 2;
    #print "005 $cpu_hz_global : $cpu_hz : $hmc/$cpu_file\n";
    last;
  }

  return $cpu_hz;
}

